/**********************************************************************
*  2005 Microchip Technology Inc.
*
* FileName:        PFC Project.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC30Fxxxx
* Compiler:        MPLAB C30 v3.00 or higher
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Incorporated ("Microchip") retains all ownership and
* intellectual property rights in the code accompanying this message and in all
* derivatives hereto.  You may use this code, and any derivatives created by
* any person or entity by or on your behalf, exclusively with Microchip,s
* proprietary products.  Your acceptance and/or use of this code constitutes
* agreement to the terms and conditions of this notice.
*
* CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS".  NO
* WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIP,S
* PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.
*
* YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE, WHETHER
* IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY),
* STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, SPECIAL,
* PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, FOR COST OR EXPENSE OF
* ANY KIND WHATSOEVER RELATED TO THE CODE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN
* ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT
* ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO
* THIS CODE, SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO
* HAVE THIS CODE DEVELOPED.
*
* You agree that you are solely responsible for testing the code and
* determining its suitability.  Microchip has no obligation to modify, test,
* certify, or support the code.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Vinaya Skanda     05/25/07  First release of source file
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
* This code is tested on dsPICDEM MC1 Motor Control Development Board
* connected to the High Voltage Power Module. The dsPIC30F device is
* used as the controller for this application */

//***************************************************************************************************************************//

// Header files //

#include "General.h"

//***************************************************************************************************************************//

volatile unsigned int PowerOnDelay = 5000;									// 125ms delay for charging caps at 40kHz sampling
volatile unsigned int SumVacLow,SumVacHigh = 0x009E;						
volatile int SumVacHighMinimum = 0x0024, SampleCountMin = 300;				// Initialize minimum values for SumVacHigh and SampleCount
volatile unsigned int AverageVac;
volatile unsigned int VpiCount=0,IrefCount=0;								// Initialize counts for running Vpi and Iref loops
volatile unsigned int SampleCount = SamplingFrequency/(2*NominalFrequency);	// Initialize sample count


//***************************************************************************************************************************//

// Main Function //

int main ( void )
{
    SetupPorts();                               // Initialize all the GPIO ports
    SetupBoard();                               // Configure the board for Reset Condition
    InitOutputCompare6();                       // Configure and intialize the Output Compare Module
    configADC();                                // Configure the ADC Module

    IFS0bits.ADIF = 0;                          // Clear ADC Interrupt Flag
    IEC0bits.ADIE = 1;                          // Enable ADC Interrupts
    OC6CONbits.OCM = 0b110;                     // Set the Output Compare Module for PWM Mode
    T3CONbits.TON = 1;                          // Start Timer-3 Module

    while(1);                                   // Loop Infinitely

}


//***************************************************************************************************************************//


// Delay Function for power-ON delay //
// 125ms delay is provided for capacitors to charge to the DC Bus Voltage //

void Delay(void)
{
    PowerOnDelay--;                 // Decrement PowerOnDelay counter
    calcVsumAndFreq();              // Calculation of Vsum and Frequency of input AC Voltage
}

//***************************************************************************************************************************//

// ADC Interrupt Service Routine //


void __attribute__((__interrupt__ , auto_psv)) _ADCInterrupt(void)

{

	
    IFS0bits.ADIF = 0;            				 // Clear ADC Interrupt Flag


    if(PowerOnDelay != 0)    					 // Check for PowerOnDelay Flag condition
    {
        Delay();             					 // Call Power-ON Delay Subroutine
    }

    else
    {

        //  if(VpiCount == 0)
        //  {
        //      VpiCount = 4;					// To run the Voltage PI controller at lower frequency
        	
				VoltagePIControl();             // Voltage Error Compensator

        //  }

        //  if(IrefCount == 0)
        //  {
        //      IrefCount = 2;					// To run the Current Reference Calculation at lower frequency
        	
				calcIacRef();                   // Calculation of Current Reference
        //  }


        CurrentPIControl();             		// Current Error Compensator
        calcVsumAndFreq();             			// Calculation of Vsum and Frequency of input AC Voltage
        pinLED1 = 1;

        //  VpiCount--;							// Decrement the Vpi calculation counter
        //  IrefCount--;						// Decrement the Iref calculation counter
    }

}

//***************************************************************************************************************************//

// Setup the HV Power Module for Reset State

void SetupBoard( void )
{
    unsigned char b;
    IEC0bits.ADIE = 0;              // Clear ADC Interrupt Flag
    pinFaultReset = 1;              // Reset all fault states to default
    for(b=0;b<10;b++)
        Nop();
    pinFaultReset = 0;              // Activate all faults
    pinPFCFire = 0;                 // Disable PFC MOSFET gate pulses
    pinBrakeFire = 0;               // Disable Brake Chopper MOSFET gate pulses
}

//***************************************************************************************************************************//

// Configure the A to D Converter

void configADC(void)

{

    ADCON1 = 0;
    ADCON1bits.FORM = 3;            // Signed Fractional Results
    ADCON1bits.SSRC = 2;            // GP Timer-3 to trigger ADC
    ADCON1bits.SIMSAM = 1;          // Simultaneous Sampling enabled
    ADCON1bits.ASAM = 1;            // Auto Sampling enabled

    ADCON2 = 0;
    ADCON2bits.CHPS = 3;            // Convert CH0, CH1, CH2 and CH3
    ADCON2bits.SMPI = 1;            // Interrupt on second sample/convert sequence
    ADCON2bits.CSCNA = 1;           // Channel Scanning enabled

    ADCON3bits.SAMC = 8;            // Auto-Sample time = 8*Tad
    ADCON3bits.ADCS = 8;            // AD Conversion time = 8*Tcy

    ADCHS = 0;
    ADCHSbits.CH123NA = 0;          // Channels CH1, CH2 and CH3 negative reference is Vref-
    ADCHSbits.CH0NA = 0;            // Channel CH0 negative reference is Vref-
    ADCHSbits.CH123SA = 0;          // Convert AN0, AN1 and AN2

    ADPCFG = 0xFFFF;
    ADPCFGbits.PCFG0 = 0;           // AN0 pin in analog mode (optional : to integrate with other applications)
    ADPCFGbits.PCFG1 = 0;           // AN1  pin in analog mode (optional : to integrate with other applications)
    ADPCFGbits.PCFG2 = 0;           // AN2(AN6) pin in analog mode
    ADPCFGbits.PCFG6 = 0;           // AN6  pin in analog mode (optional : to integrate with other applications)
    ADPCFGbits.PCFG9 = 0;           // AN9  pin in analog mode
    ADPCFGbits.PCFG11 = 0;          // AN11  pin in analog mode

    ADCSSL = 0;
    ADCSSLbits.CSSL9 = 1;           // Select AN9 for input scanning
    ADCSSLbits.CSSL11 = 1;          // Select AN11 for input scanning

    ADCON1bits.ADON = 1;            // Turn-ON the ADC Module

}

//***************************************************************************************************************************//

// Configure the Output Compare 6 Module
	/* 
	
	; Output Compare 6 and Timer 3 Configuration
	; OC6 configured for PWM mode
	; Timer 3 used as counter
	; PWM Period = (PRx + 1)*Tcy*(TMRx Prescaler)
	; PWM Period = (PR3 + 1)*33ns*(1:1)
	; OCxRS Updates PWM Duty Cycle
	; PRx gives PWM period
	
	; For a PWM frequency of 80KHz, set PRx = d#368		*/


void InitOutputCompare6(void)
{

	OC6CON = 0;						// Clear the config register
	OC6RS = 0;						// Clear the initial duty cycle register
	OC6R = 0;						// Clear the duty cycle register
	
	OC6CON = 0x0009;				// Set the operating mode for the output compare module
	PR3 = 368;						// Set the period value for the PWM pulses
	
	IFS0bits.T3IF = 0;				// Clear the timer3 interrupt flag
	IFS2bits.OC6IF = 0;				// Clear the OC6 interrupt flag
	
	IEC0bits.T3IE = 0;				// Disable timer3 interrupt
	IEC2bits.OC6IE = 0;				// Disable OC6 interrupt

}

//***************************************************************************************************************************//

// T3 Interrupt Service Routine //

void __attribute__((__interrupt__ , auto_psv)) _T3Interrupt(void)
{

IFS0bits.T3IF = 0;					// Clear timer3 interrupt flag
}

//***************************************************************************************************************************//

// OC6 Interrupt Service Routine //

void __attribute__((__interrupt__ , auto_psv)) _OC6Interrupt(void)
{

IFS2bits.OC6IF = 0;					// Clear OC6 interrupt flag

}


//***************************************************************************************************************************//

//  Math Error Trap ISR

void __attribute__((__interrupt__ , auto_psv)) _MathError(void)
{
    //  INTCON1bits.MATHERR = 0;
    while(1);
}

//***************************************************************************************************************************//

//  Address Error Trap ISR

void __attribute__((__interrupt__ , auto_psv)) _AddressError(void)

{
    //  INTCON1bits.ADDRERR = 0;
    while(1);
}

//***************************************************************************************************************************//

//  Stack Error Trap ISR

void __attribute__((__interrupt__ , auto_psv)) _StackError(void)

{
    //  INTCON1bits.STKERR = 0;
    while(1);
}

//***************************************************************************************************************************//
